////////////////////////////////////////////////////////////////////////////////
///
///		Title		: UI Framework Base Container File
///
///		Copyright (c) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: Desktop Service Development
///
////////////////////////////////////////////////////////////////////////////////
#pragma once

#include <atlcoll.h>

#include "Component.h"
#include "Layout.h"
#include "DataContext.h"

#define CHAIN_MSG_MAP_PTRMEMBER(theChainMember) \
	{ \
	if (theChainMember!=NULL) \
	if(theChainMember->ProcessWindowMessage(hWnd, uMsg, wParam, lParam, lResult)) \
	return TRUE; \
	}
/**
* @ingroup 	Base UI	
* @brief 	Base Container
* @date		10/18/2006
* @author	telltale
* @remark	
*/
class PropertyBinder;
class NPCOMMONUI_API CContainer : public CComponent
{
public:
	CContainer(void);
	virtual ~CContainer(void);

	/**
	* @brief 	set layout manager
	* @param	pLayoutManager	layout manager
	* @return	none
	*/
	void	SetLayoutManager(ILayoutManager* pLayoutManager);

	ILayoutManager* GetLayoutManager();

	DECLARE_RTTI;
	/**
	* @brief 	adjusting client size
	* @return	none
	*/
	void	RecalcLayout(BOOL bRecursive=FALSE);

	//{kr} get child list  {/kr}
	BOOL GetChildList(CAtlList<CComponent*>& List)
	{
		List.AddTailList(&m_vComponent);
		return TRUE;
	}
	//{kr} get component by name {/kr}
	CComponent* GetComponentByName(LPCTSTR pszcName);
	
	void MouseAway();

//override
	void Add(CComponent* pChild);
	BOOL Remove(CComponent* pChild);
	BOOL RemoveAll();
	void OnDraw(HDC hDC);
	void OnSize(INT32 nWidth, INT32 nHeight);
	void OnMouseLeave();
	BOOL GetPreferredSize(LPSIZE lpSize);
	BOOL GetPreferredSizeFromChild(LPSIZE lpSize);
	CComponent* FindName(LPCTSTR pszcName);
	CComponent* FindParentName(LPCTSTR pszcName);
	void GetPadding(LPRECT padding);
	void SetPadding(LPRECT padding);
	LPCTSTR GetClassName() { return _T("Container"); }
	void SetVisible(BOOL bVisible);
	void OnParentVisibleChanged(CComponent* parent, BOOL bVisible);

	virtual void DrawComponent(HDC hDC) {}
	virtual void DrawChild(HDC hDC);
	virtual void DrawNCChild(HDC hDC);
	virtual void SetDataContext(IDataContext* context);
	virtual IDataContext* GetDataContext();
	IDataContext* FindDataContext(CContainer** ppFindContainer);

	/**
	* @brief 	pretranslate message
	* @param	pMsg		massage struct
	* @return	If the msaage is processed , return the TRUE or FALSE
	*/
	virtual BOOL PreTranslateMessage(MSG* pMsg);

	CPoint ptScroll;
	void _SetScrollPos(int xPos, int yPos)
	{
		ptScroll.x = xPos;
		ptScroll.y = yPos;
	}
	BOOL _GetScrollPos(LPPOINT ppt)
	{
		ppt->x = ptScroll.x;
		ppt->y = ptScroll.y;
		return TRUE;
	}

VBEGIN_MSG_MAP(CContainer)
	if (TRUE)
	{
		/// {kr} ⺻ ޽ óϴ μ . {/kr}@n
		POSITION pos = m_vNCComponent.GetTailPosition();
		while (pos)
		{
			CComponent* pComponent = m_vNCComponent.GetPrev(pos);
			if (pComponent)
			{
				if (pComponent->IsVisible())
				{
					CHAIN_MSG_MAP_PTRMEMBER(pComponent)
				}				
			}
		}
		pos = m_vComponent.GetTailPosition();
		while (pos)
		{
			CComponent* pComponent = m_vComponent.GetPrev(pos);
			if (pComponent)
			{
				if (pComponent->IsVisible())
				{
					CHAIN_MSG_MAP_PTRMEMBER(pComponent)
				}				
			}
		}
		
		
		/// {kr} ̳ʵ ϳ Ʈ̹Ƿ ڽſ  ޽ ó . {/kr}@n
		CHAIN_MSG_MAP(CComponent)
	}
END_MSG_MAP()

protected:
	virtual void OnAddChild(CComponent* pChild) {}
	virtual void OnRemoveChild(CComponent* pChild) {}
	void AddNCComponent(CComponent* pComponent);
	BOOL RemoveNCComponent(CComponent* pComponent);
	BOOL ContainedNCComponent(CComponent* pComponent);
	
	///< child components
	CAtlList<CComponent*>	m_vComponent;
	///< non client child components
	CAtlList<CComponent*>	m_vNCComponent;
	/// {kr} Win32 콺 ޽ Ʈ {/kr}@n
	CAtlList<UINT> m_vMouseMessage;

	IDataContext* m_DataContext;

private:
	// PM-675 2007/10/20 12:37:10 
	/* {kr}
		ũѹ  콺 ø  up, down, right  콺 ,
		ũѹ ° over -> normal · ٲ ʾ.  ׸ .
	{/kr} */
	BOOL _GetAllComponent(CAtlList<CComponent*>& vComponent);
	
	/// {kr} ĸĵ Ʈ, ĸĵ Ʈ ִ . {/kr}@n
	CComponent* _GetCapturedComponent();

	friend	class CComponent;
	///< layout
	ILayoutManager*	m_pLayoutManager;	

	RECT m_Padding;
};
